import Option, { None } from '../option.ts';

// ----------------------------------------------------------------------------
// File-related types
// ----------------------------------------------------------------------------

export enum FileLang {
	C = 'C',
	CPP = 'C++',
	TypeScript = 'TypeScript',
	JavaScript = 'JavaScript',
	PHP = 'PHP',
	Go = 'Golang',
	Rust = 'Rust',
	CSS = 'CSS',
	Shell = 'Shell',
	Plain = 'Plain Text',
}

export interface HighlightingOptions {
	characters: boolean;
	numbers: boolean;
	octalNumbers: boolean;
	hexNumbers: boolean;
	binNumbers: boolean;
	jsBigInt: boolean;
	strings: boolean;
}

interface IFileType {
	readonly name: FileLang;
	readonly singleLineComment: Option<string>;
	readonly multiLineCommentStart: Option<string>;
	readonly multiLineCommentEnd: Option<string>;
	readonly keywords1: string[];
	readonly keywords2: string[];
	readonly operators: string[];
	readonly hlOptions: HighlightingOptions;
	get flags(): HighlightingOptions;
	get primaryKeywords(): string[];
	get secondaryKeywords(): string[];
	hasMultilineComments(): boolean;
}

/**
 * The base class for File Types
 */
export abstract class AbstractFileType implements IFileType {
	public readonly name: FileLang = FileLang.Plain;
	public readonly singleLineComment = None;
	public readonly multiLineCommentStart: Option<string> = None;
	public readonly multiLineCommentEnd: Option<string> = None;
	public readonly keywords1: string[] = [];
	public readonly keywords2: string[] = [];
	public readonly operators: string[] = [];
	public readonly hlOptions: HighlightingOptions = {
		characters: false,
		numbers: false,
		octalNumbers: false,
		hexNumbers: false,
		binNumbers: false,
		jsBigInt: false,
		strings: false,
	};

	get flags(): HighlightingOptions {
		return this.hlOptions;
	}

	get primaryKeywords(): string[] {
		return this.keywords1;
	}

	get secondaryKeywords(): string[] {
		return this.keywords2;
	}

	public hasMultilineComments(): boolean {
		return this.multiLineCommentStart.isSome() &&
			this.multiLineCommentEnd.isSome();
	}
}

export const defaultHighlightOptions: HighlightingOptions = {
	characters: false,
	numbers: true,
	octalNumbers: false,
	hexNumbers: false,
	binNumbers: false,
	jsBigInt: false,
	strings: true,
};
