import Option, { None, Some } from '../option.ts';
import {
	AbstractFileType,
	defaultHighlightOptions,
	FileLang,
	HighlightingOptions,
} from './base.ts';

export class CSSFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.CSS;
	public readonly singleLineComment = None;
	public readonly multiLineCommentStart: Option<string> = Some('/*');
	public readonly multiLineCommentEnd: Option<string> = Some('*/');
	public readonly keywords1 = [
		':active',
		':any-link',
		':autofill',
		':checked',
		':default',
		':disabled',
		':empty',
		':enabled',
		':first-child',
		':first-of-type',
		':focus-visible',
		':focus-within',
		':focus',
		':fullscreen',
		':hover',
		':in-range',
		':indeterminate',
		':invalid',
		':last-child',
		':last-of-type',
		':link',
		':modal',
		':nth-child',
		':nth-last-child',
		':nth-last-of-type',
		':nth-of-type',
		':only-child',
		':only-of-type',
		':optional',
		':out-of-range',
		':paused',
		':picture-in-picture',
		':placeholder-shown',
		':playing',
		':read-only',
		':read-write',
		':required',
		':root',
		':scope',
		':target',
		':user-valid',
		':valid',
		':visited',
		'::after',
		'::backdrop',
		'::before',
		'::cue',
		'::file-selector-button',
		'::first-letter',
		'::first-line',
		'::grammar-error',
		'::marker',
		'::placeholder',
		'::selection',
		'::spelling-error',
		'@charset',
		'@color-profile',
		'@container',
		'@counter-style',
		'@font-face',
		'@font-feature-values',
		'@font-palette-values',
		'@import',
		'@keyframes',
		'@layer',
		'@media',
		'@namespace',
		'@page',
		'@position-try',
		'@property',
		'@scope',
		'@starting-style',
		'@supports',
		'@view-transition',
	];
	public readonly keywords2 = [
		'animation-range-end',
		'animation-range-start',
		'accent-color',
		'animation-timeline',
		'animation',
		'animation-timing-function',
		'animation-composition',
		'animation-delay',
		'animation-direction',
		'appearance',
		'align-content',
		'animation-duration',
		'align-items',
		'animation-fill-mode',
		'align-self',
		'animation-iteration-count',
		'aspect-ratio',
		'align-tracks',
		'animation-name',
		'all',
		'animation-play-state',
		'animation-name',
		'anchor-name',
		'border-block-start-color',
		'border-inline-style',
		'backdrop-filter',
		'border-block-start-style',
		'border-inline-width',
		'backface-visibility',
		'border-block-start-width',
		'border-left',
		'background',
		'border-block-style',
		'border-left-color',
		'background-attachment',
		'border-block-width',
		'border-left-style',
		'background-blend-mode',
		'border-bottom',
		'border-left-width',
		'background-clip',
		'border-bottom-color',
		'border-radius',
		'background-color',
		'border-bottom-left-radius',
		'border-right',
		'background-image',
		'border-bottom-right-radius',
		'border-right-color',
		'background-origin',
		'border-bottom-style',
		'border-right-style',
		'background-position',
		'border-bottom-width',
		'border-right-width',
		'background-position-x',
		'border-collapse',
		'border-spacing',
		'background-position-y',
		'border-color',
		'border-start-end-radius',
		'background-repeat',
		'border-end-end-radius',
		'border-start-start-radius',
		'background-size',
		'border-end-start-radius',
		'border-style',
		'border-image',
		'border-top',
		'border-image-outset',
		'border-top-color',
		'border-image-repeat',
		'border-top-left-radius',
		'border-image-slice',
		'border-top-right-radius',
		'border-image-source',
		'border-top-style',
		'border-image-width',
		'border-top-width',
		'border-inline',
		'border-width',
		'block-size',
		'border-inline-color',
		'bottom',
		'border-inline-end',
		'border',
		'border-inline-end-color',
		'box-decoration-break',
		'border-block',
		'border-inline-end-style',
		'box-shadow',
		'border-block-color',
		'border-inline-end-width',
		'box-sizing',
		'border-block-end',
		'border-inline-start',
		'break-after',
		'border-block-end-color',
		'border-inline-start-color',
		'break-before',
		'border-block-end-style',
		'border-inline-start-style',
		'break-inside',
		'border-block-end-width',
		'border-inline-start-width',
		'border-block-start',
		'column-rule',
		'content-visibility',
		'caption-side',
		'column-rule-color',
		'column-rule-style',
		'caret-color',
		'column-rule-width',
		'column-span',
		'counter-increment',
		'column-width',
		'counter-reset',
		'columns',
		'counter-set',
		'contain',
		'contain-intrinsic-block-size',
		'clear',
		'contain-intrinsic-height',
		'clip',
		'contain-intrinsic-inline-size',
		'clip-path',
		'color',
		'contain-intrinsic-width',
		'cursor',
		'color-scheme',
		'container',
		'column-count',
		'container-name',
		'column-fill',
		'container-type',
		'column-gap',
		'content',
		'direction',
		'display',
		'empty-cells',
		'font-synthesis-position',
		'field-sizing',
		'font',
		'font-synthesis-small-caps',
		'filter',
		'font-synthesis-style',
		'font-synthesis-weight',
		'font-family',
		'font-variant',
		'font-variant-alternates',
		'font-variant-caps',
		'font-feature-settings',
		'font-variant-east-asian',
		'font-variant-emoji',
		'font-variant-ligatures',
		'font-variant-numeric',
		'font-variant-position',
		'flex',
		'font-kerning',
		'font-variation-settings',
		'flex-basis',
		'font-language-override',
		'flex-direction',
		'font-optical-sizing',
		'flex-flow',
		'font-palette',
		'font-weight',
		'flex-grow',
		'flex-shrink',
		'font-size',
		'forced-color-adjust',
		'flex-wrap',
		'font-size-adjust',
		'font-stretch',
		'float',
		'font-style',
		'font-synthesis',
		'grid-auto-columns',
		'grid-row-end',
		'gap',
		'grid-auto-flow',
		'grid-row-start',
		'grid-auto-rows',
		'grid-template',
		'grid-column',
		'grid-template-areas',
		'grid-column-end',
		'grid-template-columns',
		'grid',
		'grid-column-start',
		'grid-template-rows',
		'grid-area',
		'grid-row',
		'hanging-punctuation',
		'hyphenate-character',
		'hyphenate-limit-chars',
		'height',
		'hyphens',
		'initial',
		'inset-inline',
		'initial-letter',
		'inset-inline-end',
		'image-orientation',
		'image-rendering',
		'inline-size',
		'image-resolution',
		'inset',
		'isolation',
		'inset-area',
		'inset-block',
		'inherit',
		'inset-block-end',
		'inset-block-start',
		'justify-content',
		'justify-self',
		'justify-items',
		'justify-tracks',
		'letter-spacing',
		'list-style',
		'list-style-image',
		'line-break',
		'list-style-position',
		'line-clamp',
		'list-style-type',
		'line-height',
		'left',
		'line-height-step',
		'mask-border-outset',
		'margin',
		'mask-border-repeat',
		'margin-block',
		'mask-border-slice',
		'margin-block-end',
		'mask-border-source',
		'margin-block-start',
		'mask-border-width',
		'max-height',
		'margin-bottom',
		'mask-clip',
		'max-inline-size',
		'margin-inline',
		'mask-composite',
		'margin-inline-end',
		'mask-image',
		'max-width',
		'margin-inline-start',
		'mask-mode',
		'margin-left',
		'mask-origin',
		'margin-right',
		'mask-position',
		'min-block-size',
		'margin-top',
		'mask-repeat',
		'min-height',
		'margin-trim',
		'mask-size',
		'min-inline-size',
		'mask-type',
		'min-width',
		'masonry-auto-flow',
		'mask',
		'math-depth',
		'mix-blend-mode',
		'mask-border',
		'math-shift',
		'mask-border-mode',
		'math-style',
		'object-fit',
		'order',
		'overflow-inline',
		'object-position',
		'overflow-wrap',
		'offset',
		'orphans',
		'overflow-x',
		'offset-anchor',
		'overflow-y',
		'offset-distance',
		'outline',
		'overlay',
		'offset-path',
		'outline-color',
		'offset-position',
		'outline-offset',
		'offset-rotate',
		'outline-style',
		'overscroll-behavior',
		'outline-width',
		'overscroll-behavior-block',
		'overscroll-behavior-inline',
		'opacity',
		'overflow-anchor',
		'overscroll-behavior-x',
		'overflow-block',
		'overscroll-behavior-y',
		'overflow-clip-margin',
	];
	public readonly operators = ['::', ':', ',', '+', '>', '~', '-'];
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
	};
}
