import { node_path as path } from '../runtime/mod.ts';
import { AbstractFileType } from './base.ts';
import { CFile } from './c.ts'
import { CSSFile } from './css.ts';
import { JavaScriptFile, TypeScriptFile } from './javascript.ts';
import { ShellFile } from './shell.ts';

// ----------------------------------------------------------------------------
// External interface
// ----------------------------------------------------------------------------

export class FileType extends AbstractFileType {
	static #fileTypeMap = new Map([
		['.c', CFile],
		['.h', CFile],
		['.css', CSSFile],
		['.json', JavaScriptFile],
		['.js', JavaScriptFile],
		['.jsx', JavaScriptFile],
		['.mjs', JavaScriptFile],
		['.bash', ShellFile],
		['.sh', ShellFile],
		['.ts', TypeScriptFile],
		['.tsx', TypeScriptFile],
	]);

	public static default(): FileType {
		return new FileType();
	}

	public static from(filename: string): FileType {
		const ext = path.extname(filename);
		const type = FileType.#fileTypeMap.get(ext) ?? FileType;

		return new type();
	}
}
