import Option, { Some } from '../option.ts';
import {
	AbstractFileType,
	defaultHighlightOptions,
	FileLang,
	HighlightingOptions,
} from './base.ts';

export class JavaScriptFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.JavaScript;
	public readonly singleLineComment = Some('//');
	public readonly multiLineCommentStart: Option<string> = Some('/*');
	public readonly multiLineCommentEnd: Option<string> = Some('*/');
	public readonly keywords1 = [
		'=>',
		'await',
		'break',
		'case',
		'catch',
		'class',
		'const',
		'continue',
		'debugger',
		'default',
		'delete',
		'do',
		'else',
		'export',
		'extends',
		'false',
		'finally',
		'for',
		'function',
		'if',
		'import',
		'in',
		'instanceof',
		'let',
		'new',
		'null',
		'return',
		'static',
		'super',
		'switch',
		'this',
		'throw',
		'true',
		'try',
		'typeof',
		'var',
		'void',
		'while',
		'with',
		'yield',
	];
	public readonly keywords2 = [
		'arguments',
		'as',
		'async',
		'BigInt',
		'Boolean',
		'eval',
		'from',
		'get',
		'JSON',
		'Math',
		'Number',
		'Object',
		'of',
		'set',
		'String',
		'Symbol',
		'undefined',
	];
	public readonly operators = [
		'>>>=',
		'**=',
		'<<=',
		'>>=',
		'&&=',
		'||=',
		'??=',
		'===',
		'!==',
		'>>>',
		'+=',
		'-=',
		'*=',
		'/=',
		'%=',
		'&=',
		'^=',
		'|=',
		'==',
		'!=',
		'>=',
		'<=',
		'++',
		'--',
		'**',
		'<<',
		'>>',
		'&&',
		'||',
		'??',
		'?.',
		'?',
		':',
		'=',
		'>',
		'<',
		'%',
		'-',
		'+',
		'&',
		'|',
		'^',
		'~',
		'!',
		'.',
		',',
		';',
	];
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
		octalNumbers: true,
		hexNumbers: true,
		binNumbers: true,
		jsBigInt: true,
	};
}

export class TypeScriptFile extends JavaScriptFile {
	public readonly name: FileLang = FileLang.TypeScript;
	public readonly keywords2 = [
		...super.secondaryKeywords,
		// Typescript-specific
		'any',
		'bigint',
		'boolean',
		'enum',
		'interface',
		'keyof',
		'number',
		'private',
		'protected',
		'public',
		'string',
		'type',
		'unknown',
	];
}
