import { appendFile, readFile, writeFile } from 'node:fs/promises';
import { resolve } from 'node:path';

import { IFileIO } from './mod.ts';

export const CommonFileIO: IFileIO = {
	openFile: async function (path: string): Promise<string> {
		const filePath = resolve(path);
		const contents = await readFile(filePath, { encoding: 'utf8' });

		return contents;
	},
	appendFile: async function (path: string, contents: string): Promise<void> {
		const filePath = resolve(path);

		await appendFile(filePath, contents);
	},
	saveFile: async function (path: string, contents: string): Promise<void> {
		const filePath = resolve(path);

		await writeFile(filePath, contents);
	},
};

export default CommonFileIO;
