import { IFileIO } from '../common/runtime/mod.ts';

const DenoFileIO: IFileIO = {
	openFile: async function (path: string): Promise<string> {
		const decoder = new TextDecoder('utf-8');
		const data = await Deno.readFile(path);
		return decoder.decode(data);
	},
	appendFile: async function (path: string, contents: string): Promise<void> {
		const file = await Deno.open(path, {
			write: true,
			append: true,
			create: true,
		});
		const encoder = new TextEncoder();
		const writer = file.writable.getWriter();

		await writer.write(encoder.encode(contents));
		file.close();
	},
	saveFile: async function (path: string, contents: string): Promise<void> {
		return await Deno.writeTextFile(path, contents);
	},
};

export default DenoFileIO;
