/**
 * The main entrypoint when using Deno as the runtime
 */
import { CommonRuntime, IRuntime, RunTimeType } from '../common/runtime/mod.ts';
import DenoTerminalIO from './terminal_io.ts';
import DenoFileIO from './file_io.ts';

/**
 * The Deno Runtime implementation
 */
const DenoRuntime: IRuntime = {
	...CommonRuntime,
	name: RunTimeType.Deno,
	file: DenoFileIO,
	term: DenoTerminalIO,
	onEvent: (eventName: string, handler) =>
		globalThis.addEventListener(eventName, handler),
	onExit: (cb: () => void): void => {
		globalThis.addEventListener('onbeforeunload', cb);
		globalThis.onbeforeunload = cb;
	},
};

export default DenoRuntime;
